# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

__version__ = "2.2.0"
__author__ = "Iacopo Spalletti <i.spalletti@nephila.it>"
__all__ = ["runner"]

HELPER_FILE = "helper.py"
